package com.dt.oss;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.Date;
import java.util.UUID;

@Service

public class OSSService {

    @Value("${oss.endpoint}")
    private String endpoint;

    @Value("${oss.accessKeyId}")
    private String accessKeyId;

    @Value("${oss.accessKeySecret}")
    private String accessKeySecret;

    @Value("${oss.bucketName}")
    private String bucketName;

    // 允许的视频文件类型
    private static final String[] ALLOWED_VIDEO_TYPES = {
            "video/mp4", "video/quicktime", "video/x-msvideo",
            "video/x-ms-wmv", "video/webm", "video/3gpp"
    };

    // 允许的视频文件扩展名
    private static final String[] ALLOWED_VIDEO_EXTENSIONS = {
            ".mp4", ".mov", ".avi", ".wmv", ".webm", ".3gp"
    };

    /**
     * 上传文件到OSS（支持图片和视频）
     * @param file 上传的文件
     * @return 文件访问URL
     * @throws IOException
     */
    public String upload(MultipartFile file) throws IOException {
        // 验证文件类型
        validateFileType(file);

        // 生成唯一文件名
        String originalFilename = file.getOriginalFilename();
        String fileExtension = originalFilename.substring(originalFilename.lastIndexOf(".")).toLowerCase();
        String objectName = generateUniqueFileName(fileExtension);

        // 上传到OSS
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        try {
            ossClient.putObject(bucketName, objectName, file.getInputStream());
            return generateUrl(objectName);
        } finally {
            ossClient.shutdown();
        }
    }
    // 允许的图片文件类型
    private static final String[] ALLOWED_IMAGE_TYPES = {
            "image/jpeg", "image/png", "image/gif", "image/webp"
    };

    // 允许的图片文件扩展名
    private static final String[] ALLOWED_IMAGE_EXTENSIONS = {
            ".jpg", ".jpeg", ".png", ".gif", ".webp"
    };

    /**
     * 验证文件类型是否为允许的类型
     * @param file 上传的文件
     * @throws IOException 如果文件类型不被允许
     */
    private void validateFileType(MultipartFile file) throws IOException {
        String contentType = file.getContentType();
        String originalFilename = file.getOriginalFilename();
        String fileExtension = originalFilename.substring(originalFilename.lastIndexOf(".")).toLowerCase();

        // 检查是否为允许的视频或图片扩展名
        boolean isValidExtension = Arrays.stream(ALLOWED_VIDEO_EXTENSIONS)
                .anyMatch(ext -> ext.equals(fileExtension)) ||
                Arrays.stream(ALLOWED_IMAGE_EXTENSIONS)
                        .anyMatch(ext -> ext.equals(fileExtension));

        // 检查是否为允许的 MIME 类型
        boolean isValidContentType = contentType != null &&
                (Arrays.stream(ALLOWED_VIDEO_TYPES).anyMatch(type -> type.equals(contentType)) ||
                        Arrays.stream(ALLOWED_IMAGE_TYPES).anyMatch(type -> type.equals(contentType)));

        if (!isValidExtension || !isValidContentType) {
            throw new IOException("不支持的文件类型");
        }
    }
    /**
     * 生成唯一的文件名
     * @param fileExtension 文件扩展名
     * @return 唯一文件名
     */
    private String generateUniqueFileName(String fileExtension) {
        String uuidPart = UUID.randomUUID().toString().substring(2, 10);
        long timestamp = System.currentTimeMillis() + 1000L * 60 * 60 * 24 * 7;
        return timestamp + uuidPart + fileExtension;
    }

    /**
     * 生成文件访问URL
     * @param objectName OSS对象名称
     * @return 完整的访问URL
     */
    private String generateUrl(String objectName) {
        return "https://" + bucketName + "." + endpoint.replace("https://", "") + "/" + objectName;
    }
}